package springbootdemo;

import org.springframework.boot.context.event.*;
import org.springframework.context.*;
import org.springframework.context.event.*;
import org.springframework.stereotype.Component;

/**
 * Diese EventListener-Util-Klasse ist nur waehrend der Entwicklung sinnvoll.
 * Sie demonstriert das Abfangen einiger Events.<br>
 * <br>
 * Bei einigen Events, z.B. beim {@link SpringApplicationEvent}, werden bei der im Folgenden
 * gezeigten Art der Einbindung sehr fruehe Events nicht angezeigt, beispielsweise
 * {@link ApplicationStartedEvent}, {@link ApplicationEnvironmentPreparedEvent} und
 * {@link ApplicationPreparedEvent}.
 * Falls diese Events benoetigt werden, muss eine eigene {@link ApplicationListener}-Klasse
 * erstellt werden und ueber {@code SpringApplication.addListeners(...)} eingebunden werden.
 * @see SpringApplicationEventListenerDevUtil
 */
@Component
public class SpringEventListenerDevUtil
{
   /**
    * Da von ApplicationEvent mehrere Events abgeleitet sind, reagiert
    * diese Listener-Methode auf verschiedene Events, beispielsweise
    * {@link ContextRefreshedEvent}, {@link ApplicationReadyEvent},
    * {@link ServletRequestHandledEvent} und {@link ContextClosedEvent}
    */
   @EventListener
   public void onEvent( ApplicationEvent event )
   {
      printEventNameAndClassAndSource( "ApplicationEvent", event );
   }

   /**
    * Diese Listener-Methode reagiert nur auf ein einziges Event, naemlich auf das ContextRefreshedEvent
    */
   @EventListener
   public void onEvent( ContextRefreshedEvent event )
   {
      printEventNameAndClassAndSource( "ContextRefreshedEvent", event );
   }

   /** Anzeige von: Event-Name, Klasse und Source */
   static void printEventNameAndClassAndSource( String eventName, Object event )
   {
      String src = ( event instanceof ApplicationEvent ) ? (" von " + ((ApplicationEvent) event).getSource()) : "";
      System.out.println( "---- " + eventName + ": " + event.getClass().getSimpleName() + src );
   }
}
