package springbootdemo;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.userdetails.*;

@Configuration
public class SecurityConfig extends WebSecurityConfigurerAdapter
{
   @Autowired
   protected MeineUserRepository repo;

   /** Spezifiziert, welche HTTP-Requests eine Autorisierung mit welcher Rolle benoetigen, und den Pfad zum Login-Dialog */
   @Override
   protected void configure( HttpSecurity http ) throws Exception
   {
      http.authorizeRequests()
          .antMatchers( "/mvc-th" ).hasRole( MeineUser.MEINE_USER_ROLLENNAME )
          .antMatchers( "/mvc-th/**" ).permitAll()
          .and()
          .formLogin().loginPage( "/mvc-th/login" ).failureUrl( "/mvc-th/login?error=true" );
   }

   /** Spezifiziert, wie die Authentifizierung erfolgt, im Beispiel per JPA und DB */
   @Override
   protected void configure( AuthenticationManagerBuilder auth ) throws Exception
   {
      auth.userDetailsService( userDetailsService() );
   }

   @Bean
   @Override
   public UserDetailsService userDetailsService()
   {
      return new UserDetailsService() {
         @Override
         public UserDetails loadUserByUsername( String username ) throws UsernameNotFoundException {
            UserDetails userDetails = repo.findOne( username );
            if( userDetails != null ) {
               return userDetails;
            }
            throw new UsernameNotFoundException( "Benutzer '" + username + "' nicht vorhanden." );
         }
      };
   }
}
