package springbootdemo;

import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import java.util.List;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.SpringApplicationConfiguration;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.security.test.context.support.*;
import org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors;
import org.springframework.security.test.web.servlet.setup.SecurityMockMvcConfigurers;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.*;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

@RunWith( SpringJUnit4ClassRunner.class )
@SpringApplicationConfiguration( classes = ApplicationMain.class )
@WebAppConfiguration
public class WebMvcSecurityTest
{
   @Autowired
   private WebApplicationContext webContext;

   private MockMvc mockMvc;

   @Before
   public void setupMockMvc()
   {
      mockMvc = MockMvcBuilders.webAppContextSetup( webContext )
                               .apply( SecurityMockMvcConfigurers.springSecurity() )
                               .build();
   }

   @Test
   public void testOhneAuthentication() throws Exception
   {
      mockMvc.perform( get( "/mvc-th" ) )
             .andExpect( status().is3xxRedirection() )
             .andExpect( header().string( "Location", "http://localhost/mvc-th/login" ) );
   }

   @Test
   public void testMitFehlendemCsrfToken() throws Exception
   {
      MvcResult mr = mockMvc.perform( post( "/mvc-th" ).contentType( MediaType.APPLICATION_FORM_URLENCODED )
                                                       .param( "text", "Mein Test-Text" ) )
                            .andExpect( status().is4xxClientError() )
                            .andReturn();
      MockHttpServletResponse hsr = mr.getResponse();
      assertEquals( 403, hsr.getStatus() );
      // Je nach verwendeten Versionen kann die Fehlermeldung auch anders lauten,
      // z.B.: "Expected CSRF token not found. Has your session expired?"
      assertEquals( "Could not verify the provided CSRF token because your session was not found.", hsr.getErrorMessage() );
   }

   @Test
   @WithUserDetails( "anton" )
   @WithMockUser( roles=MeineUser.MEINE_USER_ROLLENNAME )
   public void testMitPostUndGet() throws Exception
   {
      mockMvc.perform( get( "/mvc-th" ) )
             .andExpect( status().isOk() )
             .andExpect( view().name( "MeineEntityWebseite" ) )
             .andExpect( model().attributeExists( "entitiesListe" ) )
             .andExpect( model().attribute( "entitiesListe", hasSize( 0 ) ) )
             .andExpect( content().contentType( MediaType.TEXT_HTML + ";charset=UTF-8" ) )
             .andExpect( content().string( startsWith( "<html>\n  <head>\n    <title>Spring-Boot-MVC-Thymeleaf-Webanwendung</title>" ) ) );
      mockMvc.perform( post( "/mvc-th" ).contentType( MediaType.APPLICATION_FORM_URLENCODED )
                                        .with( SecurityMockMvcRequestPostProcessors.csrf() )
                                        .param( "text", "Mein Test-Text" ) )
             .andExpect( status().is3xxRedirection() )
             .andExpect( header().string( "Location", "/mvc-th" ) );
      MvcResult mr = mockMvc.perform( get( "/mvc-th" ) )
             .andExpect( status().isOk() )
             .andExpect( view().name( "MeineEntityWebseite" ) )
             .andExpect( model().attribute( "entitiesListe", hasSize( 1 ) ) )
             .andExpect( model().attribute( "entitiesListe", hasItem( hasProperty( "id", greaterThan( Long.valueOf( 0 ) ) ) ) ) )
             .andExpect( model().attribute( "entitiesListe", hasItem( hasProperty( "datum", notNullValue() ) ) ) )
             .andExpect( model().attribute( "entitiesListe", hasItem( hasProperty( "text", equalTo( "Mein Test-Text" ) ) ) ) )
             .andReturn();
      System.out.println( "---- " + mr.getModelAndView() );
      MeineEntity me = ((List<MeineEntity>) mr.getModelAndView().getModel().get( "entitiesListe" )).get( 0 );
      assertEquals( "Mein Test-Text", me.getText() );
   }
}
